package dst.net.droid;

import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;

import dst.net.jsonObj.ItemDataNode;
import android.app.Activity;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Typeface;
import android.graphics.Paint.Align;
import android.graphics.PorterDuff.Mode;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;

public class OptionsAdapter extends BaseAdapter 
{    
	private Context mContext;
	private ItemData _folder;
	private int _multiOptionals;
	private int _idLine;
	
	public OptionsAdapter(int root, int idLine , Context c, int multiOptionals, int piece, boolean inMenu) throws UnsupportedEncodingException 
	{
		System.gc();
  
		mContext = c;
		_multiOptionals = multiOptionals;
		_idLine = idLine;
		
		try
		{
			_folder = Item.GetOptionals(root,_idLine, piece, inMenu);
		}
		catch (Exception ex)
		{
		
		}
	} 
	
	public int getCount() 
	{       
		return _folder.Nodes.size(); 
	} 
	
	public Object getItem(int position)
	{       
		return _folder.Nodes.get(position); 
	}   
	
	public long getItemId(int position)
	{     
		return _folder.Nodes.get(position).Number;
	} 
	
	public String getItemDescription(int position)
	{        
		ItemDataNode i =  _folder.Nodes.get(position);
		
		return i.Description1 + " " + i.Description2 + " " + i.Description3 + " "  + i.Description4 + " ";
	}  
	
	public double GetItemPrice (int position)
	{
		return _folder.Nodes.get(position).GetCurrentPrice();	
	}

	// create a new ImageView for each item referenced by the Adapter 
	public View getView(final int position, View convertView, ViewGroup parent) 
	{     
        final ImageView imageView;        

        Log.i("getView()", "GETTING VIEW_POSITION[" + String.valueOf(position) + "]" + (convertView != null?convertView.toString():"null")); 
 
        final ItemDataNode it =_folder.Nodes.get(position); // (ItemDataNode) getItem(position);
        
        if (convertView == null) 
        {
        	// if it's not recycled, initialize some attributes            
        	imageView = new ImageView(mContext);
        	imageView.setScaleType(ImageView.ScaleType.CENTER_CROP);            
        	imageView.setPadding(2,2,2,2);
        }  
        else  //Cached Imagest
        {            
        	imageView = (ImageView) convertView;
        }  /*************/
        
        boolean hasImage = false;
        
    	Bitmap bm = null;
    	
    	Display display = ((Activity) mContext).getWindowManager().getDefaultDisplay();  
		
		int width = (int) ((display.getWidth()* 0.65 / 3) - 4); 
		
		if (_multiOptionals!=1)
			width = (display.getWidth() / Parameters.ArticleColumns) - 4;
		
		int height = width;
    	
    	
      	if (_folder.GetImageRendered(it.Number)==null || it.Number== 0)
    	{
        	byte[] image = null;
        	
    		try {
    			
    			Resources resources = mContext.getResources();
    			Bitmap bitmap = BitmapFactory.decodeResource(resources, R.drawable.folder);
    			
    			if (it.IsFolder)
    			{
    				Bitmap bMap = drawTextToBitmap(mContext, bitmap,it.Description1,it.Description2,it.Description3,it.Description4, Color.BLACK);

    				ByteArrayOutputStream bos = new ByteArrayOutputStream();   
    				bMap.compress(CompressFormat.JPEG,80,bos);   
    				image  =  bos.toByteArray();
    			}
    			else
    			{	
    				image  = _folder.GetImageByNumber(it.Number);
    				
    				Bitmap bmp;
    				
    				if (image==null  || it.Number == 0)
    				{
    					hasImage = false;
    					// Get size from folder resource
    					bmp = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888); // this creates a MUTABLE bitmap 
    					Canvas canvas = new Canvas(bmp); 

    					RectF rect = new RectF(0,0,width,height);   
    					
    					Paint paint = new Paint();
    					paint.setStyle(Paint.Style.FILL);

    					paint.setAntiAlias(false);
    					paint.setColor(Color.parseColor(it.BackColor));
    					canvas.drawRect(rect, paint); 
    					
    					ByteArrayOutputStream bos = new ByteArrayOutputStream();
    					
    					bmp.compress(CompressFormat.JPEG,80,bos);   
    					image  =  bos.toByteArray();
    				}
    				else
    				{
    					hasImage = true;
    					
    					bmp =  Bitmap.createScaledBitmap(BitmapFactory.decodeByteArray(image,0,image.length),  width, height, true); 
    				}
    				
    				if (!hasImage)
    				{
    					bmp = drawTextToBitmap(mContext, bmp,it.Description1,it.Description2,it.Description3,it.Description4, Color.parseColor(it.TextColor));
    				}
    				else
    				{
    					bmp = drawTextToBitmapAtBottom(mContext, bmp ,it.Description1);
    				}

    				bm = getRoundedCornerBitmap(bmp,8);
    			}

    		} catch (Exception e) {
    			
    			AndroidOperations.AppendLog("OA_RemoveLine:" + e.getMessage());
    			
    			e.printStackTrace();
    		}          
    	}
		else  //Not Cached but i have rendered image
		{
			byte[] image = _folder.GetImageRendered(it.Number);
					
			bm = Bitmap.createScaledBitmap(BitmapFactory.decodeByteArray(image, 0, image.length),  width, height, true);
		}

		if (!it.IsFolder)
		{
			//draw Price always because depends on tariff
			String price = Parameters.MoneyFormatNoSymbol.format( it.GetCurrentPrice());
			
			if (it.AskPrice)
				price = "??";
        		
			if (_multiOptionals==1)
				price= "+" + price;
			
			
			if (_multiOptionals==1 && ((MultiOptionalAct) mContext).PendingFreeOptions())
					bm = drawPrice(mContext, bm ,"0,00");
			else
					bm = drawPrice(mContext, bm ,price);
		}
        	
		imageView.setImageBitmap(bm);

		imageView.setOnClickListener(new OnClickListener() {
		
		public void onClick(View v) {
			
			Animation rotate = AnimationUtils.loadAnimation(mContext, R.anim.scaledown_alpha);
			rotate.setDuration(100);
			
			if (_multiOptionals==1)
			{
				if (((MultiOptionalAct) mContext).AcceptArticleClicks)
				{
					imageView.startAnimation(rotate);
					
					((MultiOptionalAct) mContext).EventDataClick(it, imageView);
				}
			}
			else
			{
				imageView.startAnimation(rotate);
				
				((SingleOptionalAct) mContext).EventDataClick(it, imageView);
			}
		}
	});

        return imageView;
	}
	
	public Bitmap drawTextToBitmapAtBottom(Context gContext, Bitmap bitmap , String gText1) {
		
		Resources resources = gContext.getResources();
		float scale = resources.getDisplayMetrics().density;
		  
		android.graphics.Bitmap.Config bitmapConfig = bitmap.getConfig();
			  // set default bitmap config if none
			  if(bitmapConfig == null) {
			    bitmapConfig = android.graphics.Bitmap.Config.ARGB_8888;
			  }
			  // resource bitmaps are imutable, 
			  // so we need to convert it to mutable one
			  bitmap = bitmap.copy(bitmapConfig, true);

			  // new antialised Paint
			  Paint paint1 = new Paint(Paint.ANTI_ALIAS_FLAG);
			  // text color - #3D3D3D
			  paint1.setColor(Color.rgb(0, 0, 0));
			  // text size in pixels
		  
			  Typeface bold = Typeface.create(Typeface.DEFAULT_BOLD, Typeface.BOLD); 
			  paint1.setTypeface(bold);
			  
			  paint1.setTextSize((int)Parameters.SizeFont* scale);
			  // text shadow
			  /*paint1.setShadowLayer(1f, 0f, 1f, Color.WHITE);
			  paint2.setShadowLayer(1f, 0f, 1f, Color.WHITE);
			  paint3.setShadowLayer(1f, 0f, 1f, Color.WHITE);
			  paint4.setShadowLayer(1f, 0f, 1f, Color.WHITE);
			 */
			  // draw text to the Canvas center
			  Rect bounds = new Rect();
	  
			  paint1.getTextBounds(gText1, 0, gText1.length(), bounds);
			  paint1.setTypeface(Typeface.DEFAULT_BOLD); 
			  
			  int x = 3;// (bitmap.getWidth() - bounds.width())/2;

			  int y = bitmap.getHeight() - bounds.height()-5;
			  int yText = bitmap.getHeight() -2;
			  			 
			  Canvas canvas = new Canvas(bitmap);

			  Paint paint = new Paint();
			  paint.setStyle(Paint.Style.FILL);
			  paint.setAntiAlias(true);
			  paint.setColor(Color.WHITE);
			  
			  RectF rect =  new RectF(x,y ,bitmap.getWidth(),bitmap.getHeight());
				
			  if (gText1.length()>0)
				  canvas.drawRect(rect, paint);

			  canvas.drawText(gText1, x  , yText  , paint1);
			 
			  return bitmap;
	}
	
	public Bitmap drawPrice(Context gContext, Bitmap bitmap , String gText1) {
		
		
		
		Resources resources = gContext.getResources();
		float scale = resources.getDisplayMetrics().density;
		android.graphics.Bitmap.Config bitmapConfig;
		
		if (bitmap==null)
			bitmapConfig = android.graphics.Bitmap.Config.ARGB_8888;
		else
			bitmapConfig = bitmap.getConfig();
		
			  // set default bitmap config if none
			  if(bitmapConfig == null) {
			    bitmapConfig = android.graphics.Bitmap.Config.ARGB_8888;
			  }
			  // resource bitmaps are imutable, 
			  // so we need to convert it to mutable one
			 bitmap = bitmap.copy(bitmapConfig, true);

			  
			// new antialised Paint
			  Paint paint1 = new Paint(Paint.ANTI_ALIAS_FLAG);
			  paint1.setColor(Color.WHITE);
			  paint1.setTextSize((int)Parameters.SizeFont* scale);
			
			  // draw text to the Canvas center
			  Rect bounds = new Rect();
	  
			  paint1.getTextBounds(gText1, 0, gText1.length(), bounds);
			  paint1.setTextAlign(Align.RIGHT);
			  
			  int x = bitmap.getWidth() /3;
			  int xText = bitmap.getWidth() -2;
			  int y = 0;
			  int yText = bounds.height();
			  			 
			  RectF rect =  new RectF(x,y ,bitmap.getWidth() ,y + bounds.height()+2);
				
			  Canvas canvas = new Canvas(bitmap);

			  Paint paint = new Paint();
			  paint.setStyle(Paint.Style.FILL);
			  paint.setAntiAlias(true);
			  paint.setColor(Color.BLACK);
			  
			  if (gText1.length()>0)
			  {
				 canvas.drawRect(rect, paint);
				 canvas.drawText(gText1, xText  , yText  , paint1);
			  }

			  return bitmap;
	}

	public Bitmap drawTextToBitmap(Context gContext, Bitmap bitmap , String gText1, String gText2, String gText3, String gText4, int fontColor) {
		
		Resources resources = gContext.getResources();
		float scale = resources.getDisplayMetrics().density;
		  
		android.graphics.Bitmap.Config bitmapConfig = bitmap.getConfig();
			  // set default bitmap config if none
			  if(bitmapConfig == null) {
			    bitmapConfig = android.graphics.Bitmap.Config.ARGB_8888;
			  }
			  // resource bitmaps are imutable, 
			  // so we need to convert it to mutable one
			  bitmap = bitmap.copy(bitmapConfig, true);
			 
			  Paint paint1 = new Paint(Paint.ANTI_ALIAS_FLAG);
			  Paint paint2 = new Paint(Paint.ANTI_ALIAS_FLAG);
			  Paint paint3 = new Paint(Paint.ANTI_ALIAS_FLAG);
			  Paint paint4 = new Paint(Paint.ANTI_ALIAS_FLAG);
			  
			  paint1.setTextSize((int)(12)* scale);;
			  paint2.setTextSize((int)(12)* scale);
			  paint3.setTextSize((int)(12)* scale);
			  paint4.setTextSize((int)(12)* scale);
			  
			  paint1.setColor(fontColor);
			  paint2.setColor(fontColor);
			  paint3.setColor(fontColor);
			  paint4.setColor(fontColor);
			  
			  Rect bounds = new Rect();
			  Rect bounds2 = new Rect();
			  Rect bounds3 = new Rect();
			  Rect bounds4 = new Rect();
			  
			  int textHeight=  0;
			  
			  paint1.getTextBounds("Z", 0, 1, bounds);
			  
			  textHeight = bounds.height();

			  paint1.getTextBounds(gText1, 0, gText1.length(), bounds);
			  paint2.getTextBounds(gText2, 0, gText2.length(), bounds2);
			  paint3.getTextBounds(gText3, 0, gText3.length(), bounds3);
			  paint4.getTextBounds(gText4, 0, gText4.length(), bounds4);

			  Typeface bold = Typeface.create(Typeface.DEFAULT_BOLD, Typeface.BOLD); 
			  paint1.setTypeface(bold);
			  paint2.setTypeface(bold);
			  paint3.setTypeface(bold);
			  paint4.setTypeface(bold);
			  
			  int x =  (bitmap.getWidth() - bounds.width())/2;
			  int x2 = (bitmap.getWidth() - bounds2.width())/2;
			  int x3 = (bitmap.getWidth() - bounds3.width())/2;
			  int x4 = (bitmap.getWidth() - bounds4.width())/2;
			  
			  int separation = 6;

			  int y4 = bitmap.getHeight() -separation;
			  int y3 = y4 - textHeight-separation;
			  int y2 = y3 - textHeight-separation;
			  int y = y2 - textHeight-separation;
			 
			  Canvas canvas = new Canvas(bitmap);

    		  canvas.drawText(gText1, x  /** scale*/, y  , paint1);
			  canvas.drawText(gText2, x2 /** scale*/, y2 , paint2);
			  canvas.drawText(gText3, x3 /** scale*/, y3 , paint3);
			  canvas.drawText(gText4, x4 /** scale*/, y4 , paint4);
			 
			  return bitmap;
	}
	
	public Bitmap drawTextToBitmap(Context gContext, int gResId, String gText1, String gText2, String gText3, String gText4, int fontColor) {
		 
		Resources resources = gContext.getResources();
		Bitmap bitmap = BitmapFactory.decodeResource(resources, gResId);
		
		return drawTextToBitmap(gContext, bitmap,gText1, gText2, gText3, gText4 , fontColor);
	}
	
	public Bitmap getRoundedCornerBitmap(Bitmap bitmap, int pixels) { 
	        Bitmap output = Bitmap.createBitmap(bitmap.getWidth(), bitmap 
	                .getHeight(), Bitmap.Config.ARGB_8888); 
	        Canvas canvas = new Canvas(output); 
	 
	        //final int color = 0xff424242; 
	        final int color = 0xffffffff;
	        
	        final Paint paint = new Paint(); 
	        final Rect rect = new Rect(0, 0, bitmap.getWidth(), bitmap.getHeight()); 
	        final RectF rectF = new RectF(rect); 
	        final float roundPx = pixels; 
	 
	        paint.setAntiAlias(true); 
	        canvas.drawARGB(0, 0, 0, 0); 
	        paint.setColor(color); 
	        canvas.drawRoundRect(rectF, roundPx, roundPx, paint); 
	 
	        paint.setXfermode(new PorterDuffXfermode(Mode.SRC_IN)); 
	        canvas.drawBitmap(bitmap, rect, rect, paint); 
	 
	        return output; 
	    } 
}

