package dst.net.droid;

import java.util.ArrayList;

import dst.net.jsonObj.FreezeSalesOrderLine;
import dst.net.jsonObj.FreezeSalesOrderLineOption;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnLongClickListener;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.TextView;

public class RegisterMenuLinesAdapter extends ArrayAdapter<FreezeSalesOrderLine>
{ 
        private ArrayList<FreezeSalesOrderLine> items; 
        private Context _context;
        private int _selectedPos = -1;
		protected long lastTouchTime;
        private int menuColor, normalColor;
        int _resourceID ;
         boolean _menuHelper;
        
        public RegisterMenuLinesAdapter(Context context, int textViewResourceId, 
                ArrayList<FreezeSalesOrderLine> objects, boolean menuHelper) 
        { 
            super(context, textViewResourceId, objects);
            _context = context;
            this.items = objects; 
            
            lastTouchTime = 0;
            _resourceID = textViewResourceId;
            menuColor = _context.getResources().getColor(R.color.menuline);
            normalColor = _context.getResources().getColor(R.color.darkblueColor);
            _menuHelper = menuHelper;
        } 
 
        public int getPosition()
        {
        	return _selectedPos;
        }
        
        public int getSize()
        {
        	return items.size();
        }
        
        public void setSelectedPosition(int pos){
        	
        	_selectedPos = pos;
    		
        	if (!_menuHelper)
        		((MenuActionAct) _context).CurrentPosition = pos;
        	else
        		((MenuActionHelperAct) _context).CurrentPosition = pos;
        	
        	// inform the view of this change
    		notifyDataSetChanged();
    	}
        
        public void RemoveMenu (int menuNumber)
        {
        	ArrayList<FreezeSalesOrderLine> itemsToRemove = new ArrayList<FreezeSalesOrderLine>();
        	
        	for (int i=0; i< items.size(); i++)
        	{
        		FreezeSalesOrderLine temp = items.get(i);
        		
        		if (temp.Menu == menuNumber)
        		{
        			itemsToRemove.add(temp);
        		}
        	}
        	
        	for (int i=0; i< itemsToRemove.size(); i++)
        	{
        		FreezeSalesOrderLine temp = itemsToRemove.get(i);
        		
        		items.remove(temp);
        	}
        	
        }
        public void ChangeQuantityMenu(int menuNumber, double newQty)
        {
        	for (int i=0; i< items.size(); i++)
        	{
        		FreezeSalesOrderLine temp = items.get(i);
        		
        		if (temp.Menu == menuNumber)
        		{
        			temp.Quantity = newQty;
        		}
        	}	
        }
        
       // private Free
        @Override 
        public FreezeSalesOrderLine getItem(int position) { 
            return items.get(position); 
        } 
        
        @Override 
        public int getCount() { 
            return items.size(); 
        } 
  
        @Override 
        public long getItemId(int position) { 
            return items.get(position).IdLine; 
        } 

        @Override
        public View getView(final int position, View convertView, ViewGroup parent)
        { 
            View v = convertView; 
            final FreezeSalesOrderLine item =  getItem(position); 
            
            if(v == null){ 
            	   LayoutInflater vi = (LayoutInflater)_context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);  
                	v = vi.inflate(_resourceID, null);
            } 

            
            if (item != null)
            { 
                 final LinearLayout linDesc = (LinearLayout) v.findViewById(R.id.reglinDescription);
                 final TextView txtPrice = (TextView) v.findViewById(R.id.regTxtPriceLine);
                 final TextView resDescription = (TextView) linDesc.getChildAt(0);
                 final TextView resQty = (TextView) v.findViewById(R.id.textRegAmount); 
                 
                 resQty.setOnClickListener(new OnClickListener() {
     				
     				public void onClick(View v) {
     					// TODO Auto-generated method stub
     					setSelectedPosition(position);
     					
     					if (_menuHelper)
     						((MenuActionHelperAct) _context).EditAmount(item);
     					else
     						((MenuActionAct) _context).EditAmount(item);
     						
     				}
     			});
                 
                linDesc.setOnLongClickListener(new OnLongClickListener() {
					
					public boolean onLongClick(View v) {
						setSelectedPosition(position);
						
						notifyDataSetChanged();
						//((ShowRegisterAct) _context).OpenContextMenu(v);
						return true;
					}
				});
                 
                resDescription.setOnLongClickListener(new OnLongClickListener() {
 					
 					public boolean onLongClick(View v) {
 						setSelectedPosition(position);
 						
 						notifyDataSetChanged();
 						//((ShowRegisterAct) _context).OpenContextMenu(v);
 						return true;
 					}
 				});
                
                linDesc.setOnClickListener(new OnClickListener() {
					
					public void onClick(View v) {
					
						setSelectedPosition(position);
						notifyDataSetChanged();

					}
				});
                 
                resDescription.setOnClickListener(new OnClickListener() {
 					
 					public void onClick(View v) {
 					
 						setSelectedPosition(position);
 						notifyDataSetChanged();
 						
 					}
 				});
                 
                if (linDesc.getChildCount()>1)
                	linDesc.removeViews(1, linDesc.getChildCount()-1);
                
                txtPrice.setOnClickListener(new OnClickListener() {
					
					public void onClick(View v) {

						setSelectedPosition(position);
						notifyDataSetChanged();
						//((ShowRegisterAct) _context).AskArticlePrice(position);
					}
				});
                
                
                if (item.MenuPart)
                	resDescription.setBackgroundColor(menuColor);
                else
                	resDescription.setBackgroundColor(normalColor);
                
               // LinearLayout linModOpt = new LinearLayout(_context);
                
               // linModOpt.setLayoutParams(new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
                
                AddModifiers(item, linDesc);
		        AddOptions(item, linDesc);
		        
		        //linDesc.addView(linModOpt);
		        
                if (resQty != null){ 
                	resQty.setText(Parameters.QuantityFormat.format(item.Quantity)); 
                } 
 

                if (resDescription != null){ 
                	resDescription.setText(item.Description); 
                }  
                
                if (!item.Invited)
		         {
		        	 double optionsPrice = getOptionsTotal (item.Options);
		        	 
		        	 if (item.MultiOptionals==1)
		        		 txtPrice.setText(Parameters.MoneyFormatNoSymbol.format((item.Price+optionsPrice)*item.Quantity));  //Set Money Format WithoutSymbol
		        	 else
		        		 txtPrice.setText(Parameters.MoneyFormatNoSymbol.format((item.Price)*item.Quantity));  //Set Money Format WithoutSymbol
		         }
		         else
		         {
		        	 txtPrice.setText(R.string.ShowRegisterInviteLine);
		        	 //row.setBackgroundColor(Color.parseColor(Parameters.InvitedColor));
		         }
                
                if (Parameters.ECDinnerPassActive)
                {
                	final TextView txtDinerPass = (TextView)v.findViewById(R.id.regTxtDiner);
                	txtDinerPass.setText(Parameters.ECDinnerPassNames[item.CourseNumber-1]);
                	
                	txtDinerPass.setOnClickListener(new OnClickListener() {
						
						public void onClick(View v) {
							// TODO Auto-generated method stub
							setSelectedPosition(position);
							
							if (_menuHelper)
	     						((MenuActionHelperAct) _context).ChangeCourse();
	     					else
	     						((MenuActionAct) _context).ChangeCourse();
						}
					});
                }
                
                if(_selectedPos == position)
           		 v.setBackgroundColor(Color.WHITE);
           	 else
           		 v.setBackgroundColor(Color.BLACK);
                
                v.requestLayout();
            }
            return v;            
        }
        
    	private void AddModifiers(FreezeSalesOrderLine line, LinearLayout linearDesc)
		{
			//Add Modifiers
	         if (line.Modifiers!=null && line.Modifiers.length>0)
		 		{
	        	 	FlowLayout  modifiers = new FlowLayout (_context);
	        	 	FlowLayout.LayoutParams flowLP = new FlowLayout.LayoutParams(3, 3);
	        	 	
		 			for (int j=0; j<line.Modifiers.length; j++)
		 			{
		 				TextView mod = new TextView(_context);
		 				
		 				String txt = line.Modifiers[j].Description;
		 				
		 				if (txt!= null && txt.length()>3)
		 				{	txt = txt.substring(0,3);
		 				
		 					mod.setText(String.valueOf(txt));
		 				}
		 				else
		 				{
		 					mod.setText(String.valueOf(txt));
		 				}
		 				mod.setBackgroundColor(Color.YELLOW);
		 				mod.setTextColor(Color.BLACK);
		 				
		 				LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
		 				lp.setMargins(4, 0, 0, 0);
		 				mod.setLayoutParams(lp);
		 				mod.setPadding(2, 2, 2, 2);
		 				
		 				modifiers.addView(mod,flowLP );
		 			}
		 			
		 			if (modifiers.getChildCount()>0)
		 				linearDesc.addView(modifiers);
		 		}
		}
    	
    	private void AddOptions(FreezeSalesOrderLine line, LinearLayout linearDesc)
		{
			//Add Options
	         if (line.Options!=null && line.Options.length>0)
		 		{
	        	 	FlowLayout  options = new FlowLayout (_context);
		        	
	        	 	FlowLayout.LayoutParams flowLP = new FlowLayout.LayoutParams(3, 3);
		        				        	
		 			for (int j=0; j<line.Options.length; j++)
		 			{
		 				TextView opt = new TextView(_context);
		 				
		 				String txt = line.Options[j].Description;
		 				
		 				if (txt!= null && txt.length()>3)
		 				{	txt = txt.substring(0,3);
		 				
		 				opt.setText(String.valueOf(txt));
		 				}
		 				else
		 				{
		 					opt.setText(String.valueOf(line.Options[j].Description));
		 				}
		 				opt.setBackgroundColor(Color.CYAN);
		 				opt.setTextColor(Color.BLACK);
		 				
		 				LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
		 				lp.setMargins(4, 0, 0, 0);
		 				opt.setLayoutParams(lp);
		 				opt.setPadding(2, 2, 2, 2);
		 				
		 				options.addView(opt,flowLP);
		 			}
		 			if (options.getChildCount()>0)
		 				linearDesc.addView(options);
		 		}
	         
		}
		
        private double getOptionsTotal(FreezeSalesOrderLineOption[] options)
        {
        	double total = 0;
        	
        	if (options!= null)
    	    	for(int i=0; i< options.length; i++)
    	    	{
    	    		total+= options[i].Price;
    	    	}
        	return total;
        }
 }
